/**********************************************************************\
*                                                                      *
*                  Copyright (c) Exeter University,		       *
*                  Department of Computer Science,                     *
*                 (aba@uk.ac.ex.dcs) Adam Back, 1994                   *
*                                                                      *
*  GNU GPL (General Public License) Applies.                           *
*                                                                      *
\**********************************************************************/
/*
 *  Transputer sockets                        (c) Adam B '93
 *
 *  A multiplexor process runs on the transputer which routes messages to
 *  the process which started the socket server request.  A similar 
 *  multiplexor runs on the SUN socket server program.  An implicit part of 
 *  each message is then the process id of the Transputer process:
 *
 *			+----------+--------------------------------+
 * message:		| proc id  |  Message                       |
 *			+----------+--------------------------------+
 *
 * and proc id is a 32 bit integer.
 *
 *----------------------------------------------------------------------------
 *
 *	Here we define the protocol used to tell the SUN server program
 *	which socket function to call.
 *
 *  PARAMETER PASSING, BY SOCKET FUNCTION
 *
 *----------------------------------------------------------------------------
 *  SERV_socket
 *  int socket(int domain,int type,int protocol);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |  domain  |   type   | protocol |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * recv:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_listen
 *  int listen(int fd,int backlog);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  | backlog  |         fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * recv:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_accept
 *  int accept(int fd,struct sockaddr *addr,int *addrlen);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |    -     |         fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * recv:		|                  return		    |
 *			+----------+----------+----------+----------+

 *			+----------+----------+----------+----------+
 * if return != -1:	|        	    addr		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_select
 *  int select(int nfds,fd_set *readfds,fd_set *writefds,
 *	       fd_set *exceptfds, struct timeval *timeout);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |  flags   |        nfds	    |
 *			+----------+----------+----------+----------+
 *
 *			+---+---+---+---+---+---+---+---+
 * where flags=		| r | w | e | t |       -	|
 *			+---+---+---+---+---+---+---+---+
 *
 *	r = set if readfds non null
 *	w = set if writefds non null
 *	e = set if exceptfds non null
 *	t = set if timeout non null
 *
 *			+----------+----------+----------+----------+
 * if (r) send:		| 		   readfds		    |   x N
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if (w) send:		| 		  writefds		    |   x N
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if (e) send:		| 		  exceptfds		    |   x N
 *			+----------+----------+----------+----------+
 *
 * where N = (nfds + 31) / 32
 *
 *			+----------+----------+----------+----------+
 * if (t) send:		| 		timout->tv_sec		    | +
 *			+----------+----------+----------+----------+
 *			| 	       timout->tv_usec		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 * if return != -1:	
 *
 *			+----------+----------+----------+----------+
 * if (r) receive:	| 		   readfds		    |   x N
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if (w) receive:	| 		  writefds		    |   x N
 *			+----------+----------+----------+----------+
 *
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_connect
 *  int connect(int fd,const struct sockaddr *name,int namelen);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |  name ?  |         fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if name is non null:	|                   name 		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_bind
 *  int bind(int fd,const struct sockaddr *name,int namelen);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |    -     |         fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|       	    name		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_recv
 *  int recv(int fd, char *msg, int len, int flags);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |    -     |         fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		     len		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		    flags		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 * if return > 0	+----------+
 *     receive:		|   buf    |  x return
 *			+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_recvfrom
 *  int recvfrom(int fd,char* buf,int len,int flags,
 *               struct sockaddr *from,int *fromlen);
 *
 *			+----------+-----------+---------+----------+
 * send:		| command  | from null |        fd	    |
 *			+----------+-----------+---------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		     len		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		    flags		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 * if return > 0	+----------+----------+----------+----------+
 * and from is          |       	    name		    |
 * non null:		+----------+----------+----------+----------+
 *
 * if return > 0      	+----------+
 * receive:	        |   buf    |  x return
 *			+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_send
 *  int send(int fd, char *msg, int len, int flags);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |    -     |         fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		     len		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		    flags		    |
 *			+----------+----------+----------+----------+
 *
 *              	+----------+
 * send:		|   buf    |  x return
 *			+----------+
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_sendto
 *  int sendto(int fd,char *msg,int len,int flags,
 *	       struct sockaddr *to, int tolen);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |          |       fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		     len		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		    flags		    |
 *			+----------+----------+----------+----------+
 *
 * 			+----------+----------+----------+----------+
 * send:		|       	    name		    |
 * 			+----------+----------+----------+----------+
 *
 * if return > 0      	+----------+
 * receive:	        |   buf    |  x return
 *			+----------+
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_read
 *  int read(int fd,char *buf,unsigned int nbytes);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |    -     |         fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		   nbytes		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 * if return > 0	+----------+
 *     receive:		|   buf    |  x return
 *			+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_write
 *  int write(int fd,const char *buf,u_int nbytes);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |    -     |         fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * send:		|		   nbytes		    |
 *			+----------+----------+----------+----------+
 *			|   buf    |   ... 
 *			+----------+--
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 *----------------------------------------------------------------------------
 *  SERV_close
 *  int close(int fd);
 *
 *			+----------+----------+----------+----------+
 * send:		| command  |    -     |         fd	    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * receive:		|                  return		    |
 *			+----------+----------+----------+----------+
 *
 *			+----------+----------+----------+----------+
 * if return = -1:	|                  errno		    |
 *			+----------+----------+----------+----------+
 *
 */

/*
 * Using sockets at the moment
 *
 */

#define PIPE_IN "pipe_in"
#define PIPE_OUT "pipe_out"

/*
 * Socket calls available
 *
 */

#define SERV_reset 0
#define SERV_socket 1
#define SERV_listen 2
#define SERV_accept 3
#define SERV_select 4
#define SERV_connect 5
#define SERV_bind 6
#define SERV_recv 7
#define SERV_recvfrom 8
#define SERV_send 9
#define SERV_sendto 10
#define SERV_read 11
#define SERV_write 12
#define SERV_close 13
#define SERV_getsockname 14
#define SERV_gethostbyname 15
#define SERV_getpeername 16
#define SERV_shutdown 17
#define SERV_chdir 18

#define SERV_fcntl 19

/* printf for remotes uses this */

#define SERV_ss_puts 20
