/**********************************************************************\
*                                                                      *
*                  Copyright (c) Exeter University,		       *
*                  Department of Computer Science,                     *
*                 (aba@uk.ac.ex.dcs) Adam Back, 1994                   *
*                                                                      *
*  GNU GPL (General Public License) Applies.                           *
*                                                                      *
\**********************************************************************/
/*
 *  Transputer sockets                        (c) Adam B '93
 *
 *  To be used in conjunction with ProcFork().
 *
 *  ProcFork() calls RegisterFork_with_SocketLib().
 *
 *  Defines the following types:
 *
 *	struct sockaddr;
 *	struct sockaddr_un;
 *	struct sockaddr_in;
 *
 *	struct hostent;
 *
 *	fd_set;
 *
 *  The following constants:
 *
 *	SOCK_STREAM
 *	SOCK_DGRAM
 *
 *	AF_UNIX
 *	AF_INET
 *
 *	INADDR_ANY
 *
 *	FD_SETSIZE		-- user can increase from default of 256
 *
 *      int     InitSockets();  -- returns 0 on success, -1 on failure
 *
 *  And the following functions:
 *
 *	int	socket(int domain,int type,int protocol);
 *	int	listen(int fd,int backlog);
 *	int 	accept(int fd,struct sockaddr *addr,int *addrlen);
 *	int	select(int nfds,fd_set *readfds,fd_set *writefds,
 *		       fd_set *exceptfds, struct timeval *timeout);
 *	int	connect(int fd,const struct sockaddr *name,int namelen);
 *	int	bind   (int fd,const struct sockaddr *name,int namelen);
 *      int     sendto (int fd,char *msg,int len,flags,
 *                      struct sockaddr *to, int tolen);
 *      int     recv(int fd,char buf*,int len,int flags);
 *      int     recvfrom(int fd,char* buf,int len,int flags,
 *                       struct sockaddr *from,int *fromlen);
 *      int     send(int fd,char *buf,int len,int flags);
 *      int     sendto(int fd,char *buf,int len,int flags,
 *                     struct sockaddr *to,int tolen);
 *  
 *	int	read (int fd,char *buf,unsigned int nbytes);
 *	int	write(int fd,const char *buf,unsigned int nbytes);
 *	int	close(int fd);
 *
 *	int	getsockname(int fd,struct sockaddr *name,int *namelen);
 *	struct hostent *gethostbyname(const char *name);
 *
 *  And the following macros for use with select()
 *
 *	FD_SET(int fd, fd_set *fdset)
 *	FD_CLR(int fd, fd_set *fdset)
 *	FD_ISSET(int fd, fd_set *fdset)
 *	FD_ZERO(fd_set *fdset)
 */

#ifndef __Tsocket_h
#define __Tsocket_h

#include <iocntrl.h>      /* for unlink, for udp sockets */
#include "t_xdr.h"

#define EXTERN extern

struct sockaddr {
  unsigned short sa_family;
  char sa_data[14];
};

struct	sockaddr_un {
  unsigned short sun_family;
  char sun_path[108];
};

struct in_addr {
	u_long s_addr;
};

struct sockaddr_in {
	short	sin_family;
	u_short	sin_port;
	struct	in_addr sin_addr;
	char	sin_zero[8];
};

struct	hostent {
	char	*h_name;	/* official name of host */
	char	**h_aliases;	/* alias list */
	int	h_addrtype;	/* host address type */
	int	h_length;	/* length of address */
	char	**h_addr_list;	/* list of addresses from name server */
#define	h_addr	h_addr_list[0]	/* address, for backward compatiblity */
};

struct timeval {
	long	tv_sec;		/* seconds */
	long	tv_usec;	/* and microseconds */
};

#define SOCK_STREAM 1
#define SOCK_DGRAM 2

#define AF_UNIX 1
#define AF_INET 2

#define INADDR_ANY 0

#ifndef	FD_SETSIZE
#define	FD_SETSIZE 256
#endif

typedef long fd_mask;

#define NFDBITS	(sizeof(fd_mask) * 8)	/* bits per mask */
#define	howmany(x,y) (((x)+((y)-1))/(y))

typedef	struct fd_set {
  fd_mask fds_bits[howmany(FD_SETSIZE, NFDBITS)];
} fd_set;

#define	FD_SET(n, p)	((p)->fds_bits[(n)/NFDBITS] |= (1 << ((n) % NFDBITS)))
#define	FD_CLR(n, p)	((p)->fds_bits[(n)/NFDBITS] &= ~(1 << ((n) % NFDBITS)))
#define	FD_ISSET(n, p)	((p)->fds_bits[(n)/NFDBITS] & (1 << ((n) % NFDBITS)))
#define FD_ZERO(p)	bzero((char *)(p), sizeof(*(p)))

#define ENOTSOCK 1
#define MSG_PEEK 0x2

EXTERN int RegisterFork_with_SocketLib();

EXTERN int InitSockets();
EXTERN int socket(int,int,int);
EXTERN int listen(int,int);
EXTERN int accept(int,struct sockaddr *,int *);
EXTERN int select(int,fd_set *,fd_set *,fd_set *,struct timeval *);
EXTERN int connect(int,const struct sockaddr *,int);
EXTERN int bind   (int,const struct sockaddr *,int);
EXTERN int getsockname(int,struct sockaddr *,int *);
EXTERN struct hostent *gethostbyname(const char *);
EXTERN int getpeername(int, struct sockaddr*, int*);
EXTERN int shutdown(int, int);
EXTERN int Tread(int,char *,u_int);
EXTERN int Twrite(int,const char *,u_int);
EXTERN int Tclose(int);
EXTERN int recv(int,char *,int,int);
EXTERN int recvfrom(int,char*,int,int,struct sockaddr *,int *);
EXTERN int send(int,char*,int,int);
EXTERN int sendto(int,char*,int,int,struct sockaddr *,int);

EXTERN int fcntl(int,long,long);
EXTERN int chdir(char*);
EXTERN int ss_puts(char*);
/* NOT IMPLEMENTED *//*
EXTERN int recvmsg(int,struct msghdr *,int);
EXTERN int sendmsg(int,struct msghdr *,int);
*/

EXTERN Channel *inc, *outc;

#define read(fd,buf,nbytes) Tread(fd,buf,nbytes)
#define write(fd,buf,nbytes) Twrite(fd,buf,nbytes)
#define close(fd) Tclose(fd)

#endif
